unit HIDEvaMachine;

interface
uses
  Windows, Classes, Sysutils, SyncObjs, uHIDPJQDevice;
type
  TEvaluationMachine = class
  private
    fDev: THIDPJQDevice;
  public
    function setReady: boolean;
    procedure sendVoiceMessage(msgIndex: integer);
    procedure sendStarLevel(level: integer);
    function reset: Boolean;
    function getEvaluation(timeOut: integer = 10): integer;
    procedure setTimeout(timeout: Integer);
    constructor Create;
    destructor Destroy; override;
  end;
implementation

{ TEvaluationMachine }

constructor TEvaluationMachine.Create;
begin
  fDev := THIDPJQDevice.Create();
end;

destructor TEvaluationMachine.Destroy;
begin
  fDev.Free;
end;

function TEvaluationMachine.getEvaluation(timeOut: integer = 10): integer;
begin
  Result := 0;
  if Assigned(fDev) then Result := fDev.Evaluate(timeOut * 1000);
end;

function TEvaluationMachine.setReady: boolean;
begin
  result := false;
  if Assigned(fDev) then begin
    if not fDev.IsOpened() then fDev.Open();
    Result := fDev.IsOpened();
  end;
end;

procedure TEvaluationMachine.setTimeout(timeout: Integer);
begin
  // no use.
end;

procedure TEvaluationMachine.sendVoiceMessage(msgIndex: integer);
begin
  case msgIndex of
    1: fDev.Welcome();
  end;
end;



procedure TEvaluationMachine.sendStarLevel(level: integer);
begin
  if Assigned(fDev) then begin
    fDev.SetStar(Byte(level and $FF));
  end;
end;

function TEvaluationMachine.reset: Boolean;
begin
  if Assigned(fDev) then begin
    fDev.Reset();
    Result := True;
  end;
end;

end.

