#include "StdAfx.h"
#include "HIDPJQDevice.h"

// ----------------------------------------------------------------------------
// Report IDs
// ----------------------------------------------------------------------------
#define OUT_SOUNDID				0x01 
#define OUT_KEYINID 			0x02
#define OUT_SET_STARID  		0x03
#define OUT_SYS_RSTID  			0x04
#define IN_KEYCODEID 			0x05 
#define IN_STATSID			    0x06 

// ----------------------------------------------------------------------------
// Report Sizes (in bytes)
// ----------------------------------------------------------------------------
#define OUT_SOUNDSize 			 0x01
#define OUT_KEYINSize 			 0x01 
#define OUT_SYS_RSTSize 		 0x01
#define OUT_SET_STARSize 		 0x01
#define IN_KEYCODESize 		 	 0x01
#define IN_STATSSize 			 0x01 

#define WELCOME_VOICE            0x00
#define EVALUATION_VOICE         0x01
#define THANK_VOICE              0x02

CHIDPJQDevice::CHIDPJQDevice(void)
{
}

CHIDPJQDevice::~CHIDPJQDevice(void)
{
}

BOOL CHIDPJQDevice::IsConnected()
{
	return dev.GetConnectedDeviceNum(HID_VID, HID_PID);
}

BOOL CHIDPJQDevice::IsOpened() 
{
	return dev.IsOpened();
}

BYTE CHIDPJQDevice::Open()
{
	DWORD result;
	result = dev.GetConnectedDeviceNum(HID_VID, HID_PID);
	if(result > 0) {
		result = dev.Open(result - 1, HID_VID, HID_PID);
		if(result == HID_DEVICE_SUCCESS) dev.SetTimeouts(100, 100);
	}
	return (BYTE)result;
}

BYTE CHIDPJQDevice::Close()
{
	if(dev.IsOpened()) return dev.Close();
	else return 0;
}

void CHIDPJQDevice::PlaySound(BYTE index)
{
	if(dev.IsOpened()) {
		unsigned char OutputBuffer[2];
		OutputBuffer[0] = OUT_SOUNDID;
		OutputBuffer[1] = index;
		dev.SetReport_Interrupt (OutputBuffer, 2);	
	}
}

void CHIDPJQDevice::SetStar(BYTE level)
{
	if(dev.IsOpened()) {
		unsigned char OutputBuffer[2];
		OutputBuffer[0] = OUT_SET_STARID;
		OutputBuffer[1] = level;
		dev.SetReport_Interrupt (OutputBuffer, 2);	
	}
}

void CHIDPJQDevice::FlashKeyLed(UINT flash) {
	if(dev.IsOpened()) {
		unsigned char OutputBuffer[2];
		OutputBuffer[0] = OUT_KEYINID;
		OutputBuffer[1] = flash;
		dev.SetReport_Interrupt (OutputBuffer, 2);	
	}
}

BYTE CHIDPJQDevice::DoKeyIn(UINT timeout)
{
	if(dev.IsOpened()) {
		FlashKeyLed(1);	

		BYTE report[2];
		DWORD tick = GetTickCount();
		DWORD results;
		DWORD status;
		BYTE key = 0;
		while((GetTickCount() - tick) < timeout) {
			status = dev.GetReport_Interrupt(report, 2, 1, &results);
			if(status == HID_DEVICE_SUCCESS && report[0] ==	IN_KEYCODEID) {
				key = report[1];
				break;
				
			}
		}

		FlashKeyLed(0);	

		return key;

	} else return 0xFF;
}

void CHIDPJQDevice::Reset()
{
	if(dev.IsOpened()) {
		unsigned char OutputBuffer[2];
		OutputBuffer[0] = OUT_SYS_RSTID;
		OutputBuffer[1] = 0;
		dev.SetReport_Interrupt (OutputBuffer, 2);	
	}
}

BYTE CHIDPJQDevice::Status()
{
	BYTE reportbuffer[2];
	DWORD results;
	if(dev.GetReport_Interrupt(reportbuffer, 2, 1, &results))
		return reportbuffer[1];
	else return 0;
}

void CHIDPJQDevice::Welcome() {
	PlaySound(WELCOME_VOICE);
}

BYTE CHIDPJQDevice::Evaluate() {
	PlaySound(EVALUATION_VOICE);
	BYTE key = DoKeyIn(10000);
	if(key > 1 && key <= 6) Thanks();
	Reset();
	return key;
}

void CHIDPJQDevice::Thanks() {
	PlaySound(THANK_VOICE);
}