unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uHIDPJQDevice, StdCtrls, ComCtrls, ExtCtrls, uPJQDevice, uCOMPJQDevice;

type
  TMainForm = class(TForm)
    edtConnection: TEdit;
    btnConnect: TButton;
    btnDisconnect: TButton;
    btnWelcome: TButton;
    btnEvaluate: TButton;
    edtEvaluation: TEdit;
    tbStarLevel: TTrackBar;
    Label1: TLabel;
    edtStep: TEdit;
    edtCount: TEdit;
    Timer1: TTimer;
    btnAuto: TButton;
    Edit1: TEdit;
    btnUpdate: TButton;
    Memo1: TMemo;
    Button1: TButton;
    procedure btnConnectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnWelcomeClick(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure tbStarLevelChange(Sender: TObject);
    procedure btnEvaluateClick(Sender: TObject);
    procedure btnAutoClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure btnUpdateClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    pjq: IPJQDevice;
    autoCount: Integer;
    procedure CheckDevice();
    procedure Log(s: String);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
uses
  uUSBClass, SetupApi,
  com.raymon.util.strutil;

{$R *.dfm}

procedure TMainForm.btnAutoClick(Sender: TObject);
begin
  Timer1.Enabled := not Timer1.Enabled;
  if Timer1.Enabled then autoCount := 0;
end;

procedure TMainForm.btnConnectClick(Sender: TObject);
begin
  if pjq.IsConnected() then begin
    Log('device is connected');
    pjq.Open();
    if pjq.IsOpened() then begin
      edtConnection.Text := 'Connected, Active';
      Log('device is opened');
    end else begin
      Log('device cannot opened');
    end;
  end else begin
    Log('device is not connected');
  end;
end;

procedure TMainForm.btnDisconnectClick(Sender: TObject);
begin
  pjq.Close();
end;

procedure TMainForm.btnEvaluateClick(Sender: TObject);
var
  key: Byte;
begin
  key := pjq.Evaluate(5000);
  edtEvaluation.Text := Format('Key: %d', [key]);
end;

procedure TMainForm.btnUpdateClick(Sender: TObject);
var
  comPort: String;
begin
  if TUSBSerial.IsDeviceConnected('067b', '2303', comPort) then begin
    Edit1.Text := comPort;
  end else begin
    Edit1.Text := 'disconnected';
  end;
end;

procedure TMainForm.btnWelcomeClick(Sender: TObject);
begin
  pjq.Welcome();
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  devInfo: HDEVINFO;
  required: DWORD;
  len: Integer;
  devInfoData: TSPDevInfoData;
  i: Integer;
  success: Boolean;
  regType: Cardinal;
  buf: String;
  expectedDeviceId: String;
  hDeviceRegistryKey: THandle;
  comPort: String;
begin
  expectedDeviceId := UpperCase('VID_0403&PID_6001');
  devInfo := SetupDiGetClassDevs(nil, 'FTDIBUS', 0, DIGCF_PRESENT or DIGCF_ALLCLASSES);
  if (devInfo <> nil) then begin
    try
      i := 0;
      success := True;

      while (success) do begin
        required := 0;
        regType := 0;

        FillChar(devInfoData, sizeof(devInfoData), 0);
        devInfoData.cbSize := sizeof(devInfoData);
        success := SetupDiEnumDeviceInfo(devInfo, i, devInfoData);
        if success then begin
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, nil, 0, required);
          SetLength(buf, required);
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, PByte(PChar(buf)), required, required);
          buf := UpperCase(buf);
          Memo1.Lines.Add(buf);
          if StrComparison.contain(buf, expectedDeviceId) then begin
            len := 255;
            regType := 0;
            setLength(comPort, len);
            hDeviceRegistryKey := SetupDiOpenDevRegKey(devInfo, devInfoData, DICS_FLAG_GLOBAL, 0, DIREG_DEV, KEY_READ);
            if hDeviceRegistryKey <> INVALID_HANDLE_VALUE then begin
              if RegQueryValueEx(hDeviceRegistryKey, 'PortName', nil, @regType, PByte(PChar(comPort)), @len) = ERROR_SUCCESS then begin
                setLength(comPort, len - 1);
                Edit1.Text := comPort;
              end;
            end;
            break;
          end;
        end;

        Inc(i);
      end;
    finally
      SetupDiDestroyDeviceInfoList(devInfo);
    end;
  end;
end;

procedure TMainForm.CheckDevice;
begin
  if pjq.IsConnected() then begin
    edtConnection.Text := 'Connected, Idle';
  end else begin
    edtConnection.Text := 'Disconnected';
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  pjq := TPL2303PJQ.Create;
//  pjq := TSerialPJQ.Create(5);
  CheckDevice();
  autoCount := 0;
end;

procedure TMainForm.Log(s: String);
begin
  Memo1.Lines.Add(s);
end;

procedure TMainForm.tbStarLevelChange(Sender: TObject);
begin
  pjq.SetStar(tbStarLevel.Position);
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
var
  starLevel: Byte;
begin
  case Timer1.Tag of
  0:
    begin
      starLevel := Random(5);
      edtStep.Text := Format('Star [%d]', [starLevel]);
      pjq.SetStar(starLevel);
      Timer1.Tag := 1;
    end;
  1:
    begin
      edtStep.Text := 'Welcome';
      pjq.Welcome();
      Timer1.Tag := 2;
    end;
  2:
    begin
      edtStep.Text := 'Evaluate';
      pjq.Evaluate(1000);
      Timer1.Tag := 0;
      Inc(autoCount);
      edtCount.Text := Format('Count: [%d]', [autoCount]);
    end;
  end;
end;

end.
