unit uCOMPJQDevice;

interface
uses
  Windows, SysUtils, Classes, uCOMDevice, uPJQDevice;

type

  TCOMPJQDevice = class(TInterfacedObject, IReportable, IPJQDevice)
  protected
    fDev: TCOMDevice;
    function CheckResult(): Boolean;
  public
//    constructor Create(fDev: TCOMDevice);
    destructor Destroy; override;
    function IsConnected(): Boolean; virtual; abstract;
    function IsOpened(): Boolean;  virtual;
    function Open(): Boolean; virtual; abstract;
    function Close(): Integer; virtual; abstract;

    function Reset(): Boolean;
    procedure SetStar(level: Byte);
    function Status(): Byte;
    function Welcome(): Boolean;
    function Evaluate(timeout: DWORD): Byte;
  end;

  TSerialPJQ = class(TCOMPJQDevice)
  private
    fComPort: Integer;
  public
    constructor Create(comPort: Integer);
    function IsConnected(): Boolean; override;
    function IsOpened(): Boolean;  override;
    function Open(): Boolean; override;
    function Close(): Integer; override;
  end;

  TPL2303PJQ = class(TCOMPJQDevice)
  public
    constructor Create;
    function IsConnected(): Boolean; override;
    function IsOpened(): Boolean;  override;
    function Open(): Boolean; override;
    function Close(): Integer; override;
  end;

  TFTDIPJQ = class(TCOMPJQDevice)
  public
    constructor Create;
    function IsConnected(): Boolean; override;
    function IsOpened(): Boolean;  override;
    function Open(): Boolean; override;
    function Close(): Integer; override;
  end;

implementation

{ TCOMPJQDevice }

function TCOMPJQDevice.CheckResult: Boolean;
var
  b: Byte;
begin
  b := 0;
  Result := fDev.read(b, 1);
  if Result then Result := b = $33;
end;

destructor TCOMPJQDevice.Destroy;
begin
  Close();
  inherited;
end;

function TCOMPJQDevice.Evaluate(timeout: DWORD): Byte;
var
  r: Boolean;
  buf: array[0..4] of Byte;
begin
  Result := 0;
  fDev.setTimeout(timeout);
  if fDev.write('C2E', 3) then begin
    r := CheckResult();
    if r then begin
      FillChar(buf, 5, 0);
      r := fDev.read(buf, 5);
      if r then r := (buf[0] = $10) and (buf[3] = $11);
      if r then Result := buf[2] - Ord('1') + 1;
    end;
    if not r then Reset();
  end;
end;

function TCOMPJQDevice.IsOpened: Boolean;
begin
  Result := fDev.Handle <> INVALID_HANDLE_VALUE;
end;

function TCOMPJQDevice.Reset;
begin
  fDev.write('C3E', 3);
  Result := CheckResult();
end;

procedure TCOMPJQDevice.SetStar(level: Byte);
var
  cmd: array[0..2] of Char;
begin
  cmd[0] := 'C'; cmd[2] := 'E';
  cmd[1] := Char(Ord('a') + (5 - level));
  fDev.write(cmd, 3);
end;

function TCOMPJQDevice.Status: Byte;
begin
  Result := 0;
end;

function TCOMPJQDevice.Welcome;
begin
  fDev.write('C1E', 3);
  Result := CheckResult();
end;

{ TSerialPJQ }

function TSerialPJQ.Close: Integer;
begin
  FreeAndNil(fDev);
  Result := 0;
end;

constructor TSerialPJQ.Create(comPort: Integer);
begin
  fCOMPort := comPort;
  fDev := nil;
end;

function TSerialPJQ.IsConnected: Boolean;
begin
  try
    Result := Open();
    if Result then Result := Reset();
  finally
    Close();
  end;
end;

function TSerialPJQ.IsOpened: Boolean;
begin
  Result := (fDev <> nil) and (fDev.Handle <> INVALID_HANDLE_VALUE);
end;

function TSerialPJQ.Open: Boolean;
begin
  fDev := TCOMDevice.Create(fCOMPort);
  Result := fDev.Handle <> INVALID_HANDLE_VALUE;
  if Result then begin
    fDev.setState();
    fDev.setTimeout(100);
  end;
  if not Result then Close();
end;

{ TPL2303PJQ }

function TPL2303PJQ.Close: Integer;
begin
  FreeAndNil(fDev);
  Result := 0;
end;

constructor TPL2303PJQ.Create;
begin
  fDev := nil;
end;

function TPL2303PJQ.IsConnected: Boolean;
begin
  Result := TPL2303Device.GetConnected();
end;

function TPL2303PJQ.IsOpened: Boolean;
begin
  Result := (fDev <> nil) and (fDev.Handle <> INVALID_HANDLE_VALUE);
end;

function TPL2303PJQ.Open: Boolean;
begin
  Result := IsConnected();
  if Result then begin
    fDev := TPL2303Device.GetDevice();
    Result := fDev.Handle <> INVALID_HANDLE_VALUE;
    if Result then begin
      fDev.setState();
      fDev.setTimeout(100);
    end;
    if not Result then Close();
  end;
end;

{ TFTDIPJQ }

function TFTDIPJQ.Close: Integer;
begin
  FreeAndNil(fDev);
  Result := 0;
end;

constructor TFTDIPJQ.Create;
begin
  fDev := nil;
end;

function TFTDIPJQ.IsConnected: Boolean;
begin
  Result := TFTDIDevice.GetConnected();
end;

function TFTDIPJQ.IsOpened: Boolean;
begin
  Result := (fDev <> nil) and (fDev.Handle <> INVALID_HANDLE_VALUE);
end;

function TFTDIPJQ.Open: Boolean;
begin
  Result := IsConnected();
  if Result then begin
    fDev := TFTDIDevice.GetDevice();
    Result := fDev.Handle <> INVALID_HANDLE_VALUE;
    if Result then begin
      fDev.setState();
      fDev.setTimeout(100);
    end;
    if not Result then Close();
  end;
end;

end.
