unit uUSBClass;

interface
uses
  Windows, SysUtils, Classes, SetupApi;

type

  TUSBSerial = class
  public
    class function IsDeviceConnected(const vid, pid: String; var comPort: String; const Enumerator: String = 'USB'): Boolean;
  end;

  TUSBClass = class
  public
    class function IsDeviceConnected(const vid, pid: String): Boolean;
  end;
  
implementation
uses
  com.raymon.util.strutil;

{ TUSBSerial }

class function TUSBSerial.IsDeviceConnected(const vid, pid: String;
  var comPort: String; const Enumerator: String): Boolean;
var
  devInfo: HDEVINFO;
  required: DWORD;
  len: Integer;
  devInfoData: TSPDevInfoData;
  i: Integer;
  success: Boolean;
  regType: Cardinal;
  buf: String;
  expectedDeviceId: String;
  hDeviceRegistryKey: THandle;
begin
  expectedDeviceId := UpperCase('VID_' + vid + '&PID_' + pid);
  Result := False;
  devInfo := SetupDiGetClassDevs(nil, PChar(Enumerator), 0, DIGCF_PRESENT or DIGCF_ALLCLASSES);
  if (devInfo <> nil) then begin
    try
      i := 0;
      success := True;

      while (success) do begin
        required := 0;
        regType := 0;

        FillChar(devInfoData, sizeof(devInfoData), 0);
        devInfoData.cbSize := sizeof(devInfoData);
        success := SetupDiEnumDeviceInfo(devInfo, i, devInfoData);
        if success then begin
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, nil, 0, required);
          SetLength(buf, required);
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, PByte(PChar(buf)), required, required);
          buf := UpperCase(buf);
          if StrComparison.contain(buf, expectedDeviceId) then begin
            len := 255;
            regType := 0;
            setLength(comPort, len);
            hDeviceRegistryKey := SetupDiOpenDevRegKey(devInfo, devInfoData, DICS_FLAG_GLOBAL, 0, DIREG_DEV, KEY_READ);
            if hDeviceRegistryKey <> INVALID_HANDLE_VALUE then begin
              if RegQueryValueEx(hDeviceRegistryKey, 'PortName', nil, @regType, PByte(PChar(comPort)), @len) = ERROR_SUCCESS then begin
                Result := True;
                setLength(comPort, len - 1);
              end;
            end;
            break;
          end;
        end;

        Inc(i);
      end;
    finally
      SetupDiDestroyDeviceInfoList(devInfo);
    end;
  end;
end;

{ TUSBClass }

class function TUSBClass.IsDeviceConnected(const vid, pid: String): Boolean;
var
  devInfo: HDEVINFO;
  required: DWORD;
  devInfoData: TSPDevInfoData;
  i: Integer;
  success: Boolean;
  regType: Cardinal;
  buf: String;
  expectedDeviceId: String;
begin
  expectedDeviceId := UpperCase('VID_' + vid + '&PID_' + pid);
  Result := False;
  devInfo := SetupDiGetClassDevs(nil, 'USB', 0, DIGCF_PRESENT or DIGCF_ALLCLASSES);
  if (devInfo <> nil) then begin
    try
      i := 0;
      success := True;

      while (success) do begin
        required := 0;
        regType := 0;

        FillChar(devInfoData, sizeof(devInfoData), 0);
        devInfoData.cbSize := sizeof(devInfoData);
        success := SetupDiEnumDeviceInfo(devInfo, i, devInfoData);
        if success then begin
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, nil, 0, required);
          SetLength(buf, required);
          SetupDiGetDeviceRegistryProperty(devInfo, devInfoData, SPDRP_HARDWAREID, regType, PByte(PChar(buf)), required, required);
          buf := UpperCase(buf);
          if StrComparison.contain(buf, expectedDeviceId) then begin
            Result := True;
            break;
          end;
        end;

        Inc(i);
      end;
    finally
      SetupDiDestroyDeviceInfoList(devInfo);
    end;
  end;
end;

end.
